/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <regex.h>
#include <time.h>

#include <hpmud.h>

#include "device.h"
#include "logger.h"
#include "util.h"
#include "hpmuderr.h"
#include "pml.h"

static struct device * _devdup(struct device *dev);

static struct device * _devdup(struct device *dev)
{
	struct device *d = my_malloc(sizeof(*d));
	d->uri = my_malloc(strlen(dev->uri) + 1);
	strcpy(d->uri, dev->uri);
	d->model = my_malloc(strlen(dev->model) + 1);
	strcpy(d->model, dev->model);
	d->id = NULL;
	d->md = NULL;
	d->next = NULL;
	return d;
}

void device_free(struct device *d)
{
	if (d == NULL) {
		return;
	}

	device_free(d->next);

	if (d->uri != NULL) {
		my_free(d->uri);
	}
	if (d->model != NULL) {
		my_free(d->model);
	}
	if (d->id != NULL) {
		my_free(d->id);
	}
	if (d->md != NULL) {
		my_free(d->md);
	}
	free(d);
}

struct device * device_probe_devices()
{
	char buf[8192];
	enum HPMUD_RESULT res;
	int cnt, bytes_read;
	const char *pat = "direct (.*?) \"(.*?)\" \"(.*?)\" \"(.*?)\"";
	regex_t preg;
	char *s;
	struct device *firstd = NULL, *lastd = NULL;

	log_info("Probing all busses for HP devices");
	res = hpmud_probe_devices(HPMUD_BUS_ALL, buf, sizeof(buf), &cnt, &bytes_read);
	if (res != HPMUD_R_OK) {
		log_error("Can't probe devices: %s)", hpmudstrerr(res));
		return NULL;
	}

	log_info("Found %d device(s)", cnt);

	if (regcomp(&preg, pat, REG_EXTENDED | REG_ICASE) != 0) {
		log_error("Can't compile regex");
		return NULL;
	}

	s = strtok(buf, "\n");
	while (s != NULL) {
		int pres;
		size_t nmatch = 5;
		regmatch_t pmatch[5];
		struct device *d;
		char *model, *uri;

		log_debug("s: %s", s);
		pres = regexec(&preg, s, nmatch, pmatch, 0);

		if (pres != 0) {
			log_error("Invalid CUPS device string: %s", s);
			continue;
		}

		s[pmatch[1].rm_eo] = '\0';
		uri = s + pmatch[1].rm_so;
		s[pmatch[2].rm_eo] = '\0';
		model = s + pmatch[2].rm_so;

		log_debug2("uri: %s, model: %s", uri, model);

		d = (struct device *) my_malloc(sizeof(*d));
		d->uri = (char *) my_malloc(strlen(uri) + 1);
		d->model = (char *) my_malloc(strlen(model) + 1);

		strcpy(d->uri, uri);
		strcpy(d->model, model);
		d->id = NULL;
		d->md = NULL;

		if (firstd == NULL) {
			firstd = d;
		}
		if (lastd != NULL) {
			lastd->next = d;
		}
		lastd = d;

		s = strtok(NULL, "\n");
	}

	regfree(&preg);

	return firstd;
}

struct device * device_get_device(const char *uri)
{
	struct device *devices;
	struct device *device;
	struct device *sel_device = NULL;
	int i;
	int sel;

	devices = device_probe_devices();
	if (devices == NULL) {
		device_free(devices);
		return NULL;
	}

	if (uri != NULL) {
		for (device = devices; device != NULL; device = device->next) {
			if (strcmp(device->uri, uri) == 0) {
				sel_device = _devdup(device);
				device_free(devices);
				return sel_device;
			}
		}
		device_free(devices);
		return NULL;
	}

	if (devices->next == NULL) {
		sel_device = _devdup(devices);
		device_free(devices);
		return sel_device;
	}

	for (i = 1, device = devices; device != NULL; i++, device = device->next) {
		printf("%2d. %s (%s)\n", i, device->model, device->uri);
	}
	printf("Select device: ");
	scanf("%d", &sel);
	for (i = 1, device = devices; device != NULL; i++, device = device->next) {
		if (i == sel) {
			sel_device = _devdup(device);
			device_free(devices);
			return sel_device;
		}
	}

	log_error("Invalid selection");
	device_free(devices);
	exit(1);
}

int device_query(struct device *d)
{
	char attr[8192];
	enum HPMUD_RESULT res;
	int bytes_read;
	char *s, *lasts, *key, *val;
	HPMUD_DEVICE dd;

	res = hpmud_get_model_attributes(d->uri, attr, sizeof(attr), &bytes_read);
	if (res != HPMUD_R_OK) {
		log_debug("Can't get model attributes: %s", hpmudstrerr(res));
		return 1;
	}

	d->md = my_calloc(1, sizeof(*(d->md)));
	s = strtok_r(attr, "\n", &lasts);
	while (s != NULL) {
		key = strtok(s, "=");
		val = strtok(NULL, "=");

		if (strcmp(key, "io-mfp-mode") == 0) {
			d->md->io_mfp_mode = atoi(val);
		} else if (strcmp(key, "io-mode") == 0) {
			d->md->io_mode = atoi(val);
		} else if (strcmp(key, "clean-type") == 0) {
			d->md->clean_type = atoi(val);
		} else if (strcmp(key, "status-type") == 0) {
			d->md->status_type = atoi(val);
		} else if (strcmp(key, "fax-type") == 0) {
			d->md->fax_type = atoi(val);
		}

		s = strtok_r(NULL, "\n", &lasts);
	}

	res = hpmud_open_device(d->uri, d->md->io_mfp_mode, &dd);
	if (res != HPMUD_R_OK) {
		log_error("Can't open device: %s", hpmudstrerr(res));
		return 1;
	}

	res = hpmud_get_device_id(dd, attr, sizeof(attr), &bytes_read);
	if (res != HPMUD_R_OK) {
		log_error("Can't get device id: %s", hpmudstrerr(res));
		hpmud_close_device(dd);
		return 1;
	}

	d->id = my_malloc(bytes_read + 1);
	strncpy(d->id, attr, bytes_read);
	d->id[bytes_read] = '\0';

	hpmud_close_device(dd);

	return 0;
}

static int __openChannel(struct device *d, const char *service_name, HPMUD_DEVICE *dd, HPMUD_CHANNEL *cd) {
	int io_mode;
	enum HPMUD_RESULT res;

	if (d->md == NULL) {
		if (device_query(d) != 0) {
			log_debug("Can't query device");
			return 1;
		}
	}

	if (strcmp(service_name, HPMUD_S_PRINT_CHANNEL) == 0) {
		io_mode = d->md->io_mode;
		log_debug2("io-mode=%d", io_mode);
	} else {
		io_mode = d->md->io_mfp_mode;
		log_debug2("io-mfp-mode=%d", io_mode);
	}

	/* open device */
	res = hpmud_open_device(d->uri, io_mode, dd);
	if (res != HPMUD_R_OK) {
		log_error("Can't open device: %s", hpmudstrerr(res));
		return 1;
	}
	log_debug2("opened device");

	log_debug("Opening %s channel", service_name);
	res = hpmud_open_channel(*dd, service_name, cd);
	log_debug2("open_channel result: %d", res);
	if (res != HPMUD_R_OK) {
		log_error("Unable to open channel: %s", hpmudstrerr(res));
		return 1;
	}
	log_debug("channel-id=%d", *cd);
	return 0;
}

static int device_openPrint(struct device *d, HPMUD_DEVICE *dd, HPMUD_CHANNEL *cd)
{
	return __openChannel(d, HPMUD_S_PRINT_CHANNEL, dd, cd);
}

static int device_openPML(struct device *d, HPMUD_DEVICE *dd, HPMUD_CHANNEL *cd)
{
	return __openChannel(d, HPMUD_S_PML_CHANNEL, dd, cd);
}

static int __writeChannel(struct device *d, int (*opener)(struct device *, HPMUD_DEVICE *, HPMUD_CHANNEL *), const char *data, size_t len)
{
	HPMUD_DEVICE dd = -1;
	HPMUD_CHANNEL channel_id = -1;
	enum HPMUD_RESULT res;
	int bytes;
	size_t bytes_written = 0;
	int timeout = 30;
	size_t max_message_len = 8192;

	if (opener(d, &dd, &channel_id) != 0) {
		log_debug("Can't open channel");
		return 1;
	}

	log_debug("Writing channel %d...", channel_id);
	log_debug2("device=%d, channel=%d, len=%d", dd, channel_id, len);

	int bytes_to_write = MIN(len, max_message_len);
	while (bytes_written < len) {
		log_debug2("bytes_to_write=%d", bytes_to_write);
		res = hpmud_write_channel(dd, channel_id, data, bytes_to_write, timeout, &bytes);
		if (res != HPMUD_R_OK) {
			log_error("Channel write error: %s", hpmudstrerr(res));
			break;
		}
		bytes_written += bytes;
		data += bytes;
		bytes_to_write = MIN(max_message_len, (len - bytes_written));
	}

	if (bytes_written != len) {
		log_error("bytes_written != len");
	}

	hpmud_close_channel(dd, channel_id);
	hpmud_close_device(dd);
	return 0;
}

static int device_writePrint(struct device *d, const char *data, size_t len)
{
	return __writeChannel(d, device_openPrint, data, len);
}

int device_printData(struct device *d, const char *data, size_t len)
{
	return device_writePrint(d, data, len);
}

int device_setPML(struct device *d, struct pml *pml, const char *value, size_t len)
{
	HPMUD_DEVICE dd;
	HPMUD_CHANNEL ch;
	char buf[256];
	char oid[64];
	size_t len2;
	enum HPMUD_RESULT res;
	int pml_result;

	log_debug("value:");
	log_data(value, len);

	if (device_openPML(d, &dd, &ch) != 0) {
		log_debug("Can't open channel");
		return 1;
	}

	len2 = pml_ConvertToPMLDataFormat(value, len, pml->type, buf, sizeof(buf));
	pml_PMLToSNMP(pml->oid, oid, sizeof(oid));
	log_debug2("oid: %s", oid);

	log_debug("converted data");
	log_data(buf, len2);

	res = hpmud_set_pml(dd, ch, oid, pml->type, buf, len2, &pml_result);

	hpmud_close_channel(dd, ch);
	hpmud_close_device(dd);
	return pml_result;
}

int device_getPML(struct device *d, struct pml *pml, char *out, size_t *size)
{
	HPMUD_DEVICE dd;
	HPMUD_CHANNEL ch;
	char buf[256];
	char oid[64];
	enum HPMUD_RESULT res;
	int bytes_read;
	int type;
	int pml_result;

	if (device_openPML(d, &dd, &ch) != 0) {
		log_debug("Can't open channel");
		return 1;
	}

	pml_PMLToSNMP(pml->oid, oid, sizeof(oid));
	log_debug2("oid: %s", oid);
	res = hpmud_get_pml(dd, ch, oid, buf, sizeof(buf), &bytes_read, &type, &pml_result);
	log_debug("data");
	log_data(buf, bytes_read);

	*size = pml_ConvertFromPMLDataFormat(buf, bytes_read, type, out, *size);
	log_debug("converted data");
	log_data(out, *size);

	hpmud_close_channel(dd, ch);
	hpmud_close_device(dd);
	return pml_result;
}
