/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>

#include "hp-clean.h"
#include "logger.h"
#define SELF "hp-clean"
#define VERSION "0.1.1"
#include "util.h"
#include "device.h"
#include "maint.h"

int clean(const char *device_uri, int clean_level, int mode)
{
	struct device *device;
	int clean_type;
	const char *host = "localhost";
	const int port = 9100;
	int ret = 1;

	/* get device */
	device = device_get_device(device_uri);
	if (device == NULL) {
		if (device_uri == NULL) {
			log_error("Can't find any supported HP devices.");
			return 2;
		} else {
			log_error("Can't find device `%s'", device_uri);
			return 3;
		}
	}
	log_info("Using device %s", device->uri);

	if (device_query(device) != 0) {
		log_error("Can't query device");
		return -1;
	}
	clean_type = device->md->clean_type;
	log_debug("clean-type: %d", clean_type);

	if (device_uri == NULL) {
		fprintf(stderr, "%s (%s)\n", device->model, device->uri);
	}

	switch (clean_type) {
		case CLEAN_TYPE_PCL:
			ret = maint_clean_pcl(clean_level, mode, device, host, port);
			break;
		case CLEAN_TYPE_LIDIL:
			ret = maint_clean_lidil(clean_level, mode, device, host, port);
			break;
		case CLEAN_TYPE_PCL_WITH_PRINTOUT:
			log_error("Cleaning not yet implemented for this device.");
			ret = 5;
			break;
		default:
			log_error("Cleaning not needed or supported on this device.");
			ret = 4;
	}

	if (ret == 0) {
		printf("Level %d cleaning started. Wait till it completes.",
				clean_level);
	}

	/* clean up */
	device_free(device);

	return ret;
}

static void help(const char *pname)
{
	printf("Usage: %s [OPTION]...\n"
			"Cartridge cleaning utility for HPLIP supported inkjet printers.\n"
			"\n"
			"  -d, --device=URI          use device with URI URI\n"
			"  -c, --clean-level=LEVEL   use clean level LEVEL {1, 2, 3}\n"
			"  -m, --mode=MODE           use mode MODE {printserv, hpmud}\n"
			"  -l, --log-level=LEVEL     set log level to LEVEL\n"
#ifndef NDEBUG
			"                            {none, error, warning, debug, debug2, all}\n"
#else
			"                            {none, error, all}\n"
#endif
			"      --help                display this help and exit\n"
			"      --version             output version information and exit\n",
			pname);
}

int main(int argc, char **argv)
{
	int c;
	char *device_uri = NULL;
	int clean_level = -1;
	int clean_set = 0;
	int mode = MODE_HPMUD;

	log_program = SELF;
	log_level = LOG_LEVEL_ERROR;

	while (1) {
		static struct option long_options[] = {
			{ "device",			required_argument,	0, 'd' },
			{ "clean-level",	required_argument,	0, 'c' },
			{ "mode",			required_argument,	0, 'm' },
			{ "log-level",  	required_argument,  0, 'l' },
            { "help",       	no_argument,        0, 250 },
            { "version",    	no_argument,        0, 251 },
			{ 0, 0, 0, 0 }
		};
		int option_index = 0;

		c = getopt_long(argc, argv, "d:c:m:l:", long_options, &option_index);

		if (c == -1) {
			break;
		}

		switch (c) {
			case 'd':
				device_uri = optarg;
				break;
			case 'c':
				clean_set = 1;
				clean_level = atoi(optarg);
				if (clean_level < 1 || clean_level > 3) {
					helpinfo(argv[0], "illegal clean level `%s'", optarg);
					return 1;
				}
				break;
			case 'm':
				if (strcmp(optarg, "hpmud") == 0) {
					mode = MODE_HPMUD;
				} else if (strcmp(optarg, "printserv") == 0) {
					mode = MODE_PRINTSERV;
				} else {
					helpinfo(argv[0], "illegal mode `%s'", optarg);
					return 1;
				}
				break;
			case 'l':
				if (log_parse(optarg, &log_level) != 0) {
					helpinfo(argv[0], "illegal log level `%s'", optarg);
					return 1;
				}
				break;
			case 250:
				help(argv[0]);
				return 0;
			case 251:
				print_version();
				return 0;
			case '?':
				helpinfo(argv[0], NULL);
				return 1;
		}
	}

	if (optind < argc) {
		helpinfo(argv[0], "extra operand `%s'", argv[optind]);
		return 1;
	}

	if (!clean_set) {
		helpinfo(argv[0], "no clean level specified");
		return 1;
	}

	print_copy_msg();

	return clean(device_uri, clean_level, mode);
}
