/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>

#include "logger.h"
#define SELF "hp-faxsetup"
#define VERSION "0.1.0"
#include "util.h"
#include "device.h"
#include "pmlfax.h"

static int read(char *s, size_t size)
{
	char c;
	size_t i = 0;

	while ((c = getchar()) != '\n' && i < size - 1) {
		s[i++] = c;
	}
	s[i] = '\0';
	return i;
}

static int doit(struct device *d)
{
	char buf[256];

	if (pmlfax_getStationName(d, buf, sizeof(buf)) != 0) {
		return 1;
	}
	printf("Your name [%s]: ", buf);
	if (read(buf, sizeof(buf)) > 0) {
		if (pmlfax_setStationName(d, buf) != 0) {
			return 1;
		}
	}

	if (pmlfax_getPhoneNum(d, buf, sizeof(buf)) != 0) {
		return 1;
	}
	printf("Your fax number [%s]: ", buf);
	if (read(buf, sizeof(buf)) > 0) {
		if (pmlfax_setPhoneNum(d, buf) != 0) {
			return 1;
		}
	}

	return 0;
}

static int faxsetup(const char *device_uri)
{
	struct device *device;
	int ret = 1;
	int fax_type;

	/* get device */
	device = device_get_device(device_uri);
	if (device == NULL) {
		if (device_uri == NULL) {
			log_error("Can't find any supported HP devices.");
			return 2;
		} else {
			log_error("Can't find device `%s'", device_uri);
			return 3;
		}
	}
	log_info("Using device %s", device->uri);

	if (device_query(device) != 0) {
		log_error("Can't query device");
		ret = -1;
		goto out;
	}
	fax_type = device->md->fax_type;
	log_debug("fax-type: %d", fax_type);

	if (device_uri == NULL) {
		fprintf(stderr, "%s (%s)\n", device->model, device->uri);
	}

	switch (fax_type) {
		case FAX_TYPE_BLACK_SEND_EARLY_OPEN:
		case FAX_TYPE_BLACK_SEND_LATE_OPEN:
			ret = doit(device);
			if (ret != 0) {
				log_error("An error occurred");
			}
			break;
		case FAX_TYPE_SOAP:
			log_error("Fax setup not yet implemented for this device.");
			ret = 5;
			break;
		case FAX_TYPE_NONE:
		default:
			log_error("Device does not support fax setup.");
			ret = 4;
	}

out:
	/* clean up */
	device_free(device);

	return ret;
}

static void help(const char *pname)
{
	printf("Usage: %s [OPTION]...\n"
			"Setup fax settings on an HP Officejet.\n"
			"\n"
			"  -d, --device=URI          use device with URI URI\n"
			"  -l, --log-level=LEVEL     set log level to LEVEL\n"
#ifndef NDEBUG
			"                            {none, error, warning, debug, debug2, all}\n"
#else
			"                            {none, error, all}\n"
#endif
			"      --help                display this help and exit\n"
			"      --version             output version information and exit\n",
			pname);
}

int main(int argc, char **argv)
{
	int c;
	char *device_uri = NULL;

	log_program = SELF;
	log_level = LOG_LEVEL_ERROR;

	while (1) {
		static struct option long_options[] = {
			{ "device",			required_argument,	0, 'd' },
			{ "log-level",  	required_argument,  0, 'l' },
            { "help",       	no_argument,        0, 250 },
            { "version",    	no_argument,        0, 251 },
			{ 0, 0, 0, 0 }
		};
		int option_index = 0;

		c = getopt_long(argc, argv, "d:l:", long_options, &option_index);

		if (c == -1) {
			break;
		}

		switch (c) {
			case 'd':
				device_uri = optarg;
				break;
			case 'l':
				if (log_parse(optarg, &log_level) != 0) {
					helpinfo(argv[0], "illegal log level `%s'", optarg);
					return 1;
				}
				break;
			case 250:
				help(argv[0]);
				return 0;
			case 251:
				print_version();
				return 0;
			case '?':
				helpinfo(argv[0], NULL);
				return 1;
		}
	}

	if (optind < argc) {
		helpinfo(argv[0], "extra operand `%s'", argv[optind]);
		return 1;
	}

	print_copy_msg();

	return faxsetup(device_uri);
}
