/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <getopt.h>

/* for getting terminal width */
#include <sys/ioctl.h>
#include <unistd.h>

#include <hpmud.h>

#include "hp-levels.h"
#include "logger.h"
#define SELF "hp-levels"
#define VERSION "0.2.3"
#include "util.h"
#include "device.h"
#include "status.h"

void print_graph(int level, int width)
{
	int i,j;

	width -= 16;
	for (j = 0; j < 3; j++) {
		if (j == 1) {
			printf("|");
		} else {
			printf("+");
		}
		if (j == 1) {
			int lvl = 0;
			if (level != 0) {
				lvl = (int) ((level/100.0)*(width-2));
				for (i = 0; i < lvl; i++) {
					printf("#");
				}
			}
			for (i = lvl+1; i < width-1; i++) {
				printf(" ");
			}
		} else {
			for (i = 1; i < width-1; i++) {
				printf("-");
			}
		}
		if (j == 1) {
			printf("| (approx. %3d%%)\n", level);
		} else {
			printf("+\n");
		}
	}
}

int print_levels(const char *device_uri, int width, int display_style)
{
	struct device *device;
	int i;
	struct status_block * status_block;
	const char *agent_type_strings[] = { AGENT_TYPE_STRINGS };
	const char *agent_kind_strings[] = { AGENT_KIND_STRINGS };
	const char *agent_level_trigger_strings[] = { AGENT_LEVEL_TRIGGER_STRINGS };

	if (device_uri == NULL && (display_style == 2 || display_style == 4)) {
		return 1;
	}

	/* get device */
	device = device_get_device(device_uri);
	if (device == NULL) {
		if (device_uri == NULL) {
			log_error("Can't find any supported HP devices.");
			return 2;
		} else {
			log_error("Can't find device `%s'", device_uri);
			return 3;
		}
	}
	log_info("Using device %s", device->uri);

	if (device_query(device) != 0) {
		log_error("Can't query device");
		return -1;
	}
	log_debug("device id: %s", device->id);

	if (device->md->status_type == HPMUD_STATUSTYPE_NA) {
		log_error("Status unsupported on device `%s'", device->uri);
		return 4;
	}
	/* TODO add support for other status reporting methods */
	if (device->md->status_type != HPMUD_STATUSTYPE_SFIELD
			&& device->md->status_type != HPMUD_STATUSTYPE_VSTATUS) {
		log_error("Status not yet implemented for device `%s'. Bummer :/",
				device->model);
		return 5;
	}

	if (device_uri == NULL && display_style != 2 && display_style != 4) {
		printf("%s (%s)\n", device->model, device->uri);
	}

	/* print status */
	if (display_style == 2) {
		printf("<table>\n");
	}
	status_block = status_parseStatus(device->id);
	for (i = 0; i < status_block->num_agents; i++) {
		struct agent *agent = &(status_block->agents[i]);
		const char *type_str, *kind_str, *level_trigger_str;
		int foo;
		if (agent->type >= 0 && agent->type <= 19) {
			type_str = agent_type_strings[agent->type];
		} else {
			type_str = AGENT_TYPE_UNKNOWN_STRING;
		}
		if (agent->kind >= 0 && agent->kind <= 9) {
			kind_str = agent_kind_strings[agent->kind];
		} else {
			kind_str = AGENT_KIND_UNKNOWN_STRING;
		}
		if (agent->level_trigger >= 0 && agent->level_trigger <= 7) {
			level_trigger_str = agent_level_trigger_strings[agent->level_trigger];
		} else {
			level_trigger_str = AGENT_LEVEL_TRIGGER_UNKNOWN_STRING;
		}
		/* TODO: switch kind (ink, toner, head, battery, ... */
		foo = (agent->level_trigger == 6 || agent->level_trigger == 7);
		if (display_style == 1) {
			printf("%s %s:\t%3d%% (%s%s)\n",
					type_str,
					kind_str, agent->level,
					foo?level_trigger_str:agent->level_trigger==5?"ink level ":"ink level is ",
					foo?" of ink":level_trigger_str);
		} else if (display_style == 2) {
			printf("\t<tr><td>%s %s</td><td>%s%s</td><td style=\"text-align: right;\">%d%%</td></tr>\n",
					type_str,
					kind_str,
					foo?level_trigger_str:agent->level_trigger==5?"ink level ":"ink level is ",
					foo?" of ink":level_trigger_str,
					agent->level);
		} else if (display_style == 4) {
			printf("%d %d %d %d %d %d\n", agent->id, agent->type, agent->kind,
					agent->level, agent->level_trigger, agent->health);
		} else {
			printf("%s %s: (%s%s)\n",
					type_str,
					kind_str,
					foo?level_trigger_str:agent->level_trigger==5?"ink level ":"ink level is ",
					foo?" of ink":level_trigger_str);
			print_graph(agent->level, width);
			printf("\n");
		}
	}
	if (display_style == 2) {
		printf("</table>\n");
	}

	/* clean up */
	my_free(status_block->agents);
	my_free(status_block);
	device_free(device);

	return 0;
}

int get_term_width(int default_width) {
	struct winsize win;
	int width;

	if (ioctl(STDIN_FILENO, TIOCGWINSZ, &win)) {
		/* ioctl failed */
		width = default_width;
	} else {
		width = win.ws_col;
	}
	log_debug2("width: %d", width);
	return width;
}

void help(const char *pname)
{
	printf("Usage: %s [OPTION]...\n"
			"Display bar graphs of current supply levels for supported HPLIP printers.\n"
			"\n"
			"  -d, --device=URI          use device with URI URI\n"
			"  -n, --no-bar-graphs       do not display bar graphs\n"
			"      --html                generate html output\n"
			"      --web-interface       generate output for hp-utils web interface\n"
			"  -l, --log-level=LEVEL     set log level to LEVEL\n"
#ifndef NDEBUG
			"                            {none, error, warning, debug, debug2, all}\n"
#else
			"                            {none, error, all}\n"
#endif
			"      --help                display this help and exit\n"
			"      --version             output version information and exit\n",
			pname);
}

int main(int argc, char **argv)
{
	int c;
	char *device_uri = NULL;
	int display_style = 0;

	log_program = SELF;
	log_level = LOG_LEVEL_ERROR;

	while (1) {
		static struct option long_options[] = {
			{ "device",			required_argument,	0, 'd' },
			{ "no-bar-graphs",	no_argument,		0, 'n' },
			{ "html",			no_argument,		0, 128 },
			{ "web-interface",	no_argument,		0, 130 },
			{ "log-level",  	required_argument,  0, 'l' },
            { "help",       	no_argument,        0, 250 },
            { "version",    	no_argument,        0, 251 },
			{ 0, 0, 0, 0 }
		};
		int option_index = 0;

		c = getopt_long(argc, argv, "d:nl:", long_options, &option_index);

		if (c == -1) {
			break;
		}

		switch (c) {
			case 'd':
				device_uri = optarg;
				break;
			case 'n':
				display_style = 1;
				break;
			case 128:
				display_style = 2;
				break;
			case 130:
				display_style = 4;
				break;
			case 'l':
				if (log_parse(optarg, &log_level) != 0) {
					helpinfo(argv[0], "illegal log level `%s'", optarg);
					return 1;
				}
				break;
			case 250:
				help(argv[0]);
				return 0;
			case 251:
				print_version();
				return 0;
			case '?':
				helpinfo(argv[0], NULL);
				return 1;
		}
	}

	if (optind < argc) {
		helpinfo(argv[0], "extra operand `%s'", argv[optind]);
		return 1;
	}

	if (display_style != 2 && display_style != 4) {
		print_copy_msg();
	}

	return print_levels(device_uri, get_term_width(80), display_style);
}
