/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <errno.h>

#include <unistd.h>
#include <sys/socket.h>
#include <netinet/ip.h>

#include "logger.h"
#define SELF "hp-printserv"
#define VERSION "0.1.0"
#include "util.h"
#include "device.h"
#include "hpmuderr.h"

#define DEFAULT_PORT 9100
#define BUFSZ 4096

static int process(int sock, const char *uri, int io_mode) {
	enum HPMUD_RESULT res;
	HPMUD_DEVICE dd;
	HPMUD_CHANNEL cd;
	char buf[BUFSZ];
	int bytes = 0;
	int bytes_written;
	int state = 0;
	char job_name[256];

	log_debug("job start");

	res = hpmud_open_device(uri, io_mode, &dd);
	if (res != HPMUD_R_OK) {
		log_error("hpmud_open_device: %s", hpmudstrerr(res));
		return -1;
	}
	log_debug("Opened device");
	res = hpmud_open_channel(dd, HPMUD_S_PRINT_CHANNEL, &cd);
	if (res != HPMUD_R_OK) {
		log_error("hpmud_open_channel: %s", hpmudstrerr(res));
		return -1;
	}
	log_debug("Opened channel %s", HPMUD_S_PRINT_CHANNEL);

	while ((bytes = recv(sock, buf, sizeof(buf), 0)) != 0) {
		int i;
		int start = -1;
		int end = -1;

		res = hpmud_write_channel(dd, cd, buf, bytes, 30, &bytes_written);
		if (res != HPMUD_R_OK) {
			log_error("hpmud_write_channel: %s", hpmudstrerr(res));
			break;
		}

		for (i = 0; i < bytes; i++) {
			if (buf[i] == '@') {
				if (start == -1) {
					start = i;
				} else {
					end = i;
					break;
				}
			}
		}

		if (state == 0 && end != -1) {
			state = 1;
			strncpy(job_name, buf+start, end-start);
			log_info("job name: %s", job_name);
		}

		bytes_written = bytes;
		log_debug("bytes read: %d, bytes written: %d", bytes, bytes_written);
	}
	log_debug("job end");

	hpmud_close_channel(dd, cd);
	hpmud_close_device(dd);
	return 0;
}

static int printserv(const char *uri, int port)
{
	struct device *d;
	int io_mode;
	int sock;
	struct sockaddr_in sa;
	int yes=1;
	//char yes='1'; // Solaris people use this

	d = device_get_device(uri);
	if (d == NULL) {
		if (uri == NULL) {
			log_error("Can't find any supported HP devices.");
			return 2;
		} else {
			log_error("Can't find device `%s'", uri);
			return 3;
		}
	}
	log_info("Using device %s", d->uri);

	if (device_query(d) != 0) {
		log_error("Can't query device");
		return -1;
	}
	io_mode = d->md->io_mode;
	log_debug("io-mode: %d", io_mode);

	if (uri == NULL) {
		fprintf(stderr, "%s (%s)\n", d->model, d->uri);
	}

	if ((sock = socket(PF_INET, SOCK_STREAM, IPPROTO_IP)) == -1) {
		log_error("socket error: %s", strerror(errno));
		return 1;
	}

	memset(&sa, 0, sizeof(sa));
	sa.sin_family = AF_INET;
	sa.sin_port = htons(port);
	sa.sin_addr.s_addr = htonl(INADDR_ANY);

	// lose the pesky "Address already in use" error message
	if (setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(int)) == -1) {
		log_error("setsockopt: %s", strerror(errno));
		return 1;
	}

	if (bind(sock, (struct sockaddr *) &sa, sizeof(sa)) == -1) {
		log_error("socket error: %s", strerror(errno));
		return 1;
	}

	if (listen(sock, 10) == -1) {
		log_error("socket error: %s", strerror(errno));
		return 1;
	}

	for ( ; ; ) {
		int cl_sock;
		cl_sock = accept(sock, NULL, NULL);

		if (cl_sock < 0) {
			log_error("accept failed");
			continue;
		}

		/* TODO fork */
		process(cl_sock, d->uri, io_mode);
		
		shutdown(cl_sock, 2);
		close(cl_sock);
	}
	
	device_free(d);
	return 0;
}

static void help(const char *pname)
{
	printf("Usage: %s [OPTION]...\n"
			"Simple print server.\n"
			"\n"
			"  -d, --device=URI          use device with URI URI\n"
			"  -p, --port=PORT           listen on port PORT\n"
			"  -l, --log-level=LEVEL     set log level to LEVEL\n"
#ifndef NDEBUG
			"                            {none, error, warning, debug, debug2, all}\n"
#else
			"                            {none, error, all}\n"
#endif
			"      --help                display this help and exit\n"
			"      --version             output version information and exit\n",
			pname);
}

int main(int argc, char **argv)
{
	int c;
	char *device_uri = NULL;
	int port = DEFAULT_PORT;

	log_program = SELF;
	log_level = LOG_LEVEL_ERROR;

	while (1) {
		static struct option long_options[] = {
			{ "device",			required_argument,	0, 'd' },
			{ "port",			required_argument,	0, 'p' },
			{ "log-level",  	required_argument,  0, 'l' },
            { "help",       	no_argument,        0, 250 },
            { "version",    	no_argument,        0, 251 },
			{ 0, 0, 0, 0 }
		};
		int option_index = 0;

		c = getopt_long(argc, argv, "d:p:l:", long_options, &option_index);

		if (c == -1) {
			break;
		}

		switch (c) {
			case 'd':
				device_uri = optarg;
				break;
			case 'p':
				port = atoi(optarg);
				break;
			case 'l':
				if (log_parse(optarg, &log_level) != 0) {
					helpinfo(argv[0], "illegal log level `%s'", optarg);
					return 1;
				}
				break;
			case 250:
				help(argv[0]);
				return 0;
			case 251:
				print_version();
				return 0;
			case '?':
				helpinfo(argv[0], NULL);
				return 1;
		}
	}

	if (optind < argc) {
		helpinfo(argv[0], "extra operand `%s'", argv[optind]);
		return 1;
	}

	print_copy_msg();

	return printserv(device_uri, port);
}
