/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

#include <stdio.h>
#include <getopt.h>
#include <hpmud.h>

#include "hp-status.h"
#include "logger.h"
#define SELF "hp-status"
#define VERSION "0.1.0"
#include "util.h"
#include "device.h"
#include "status.h"

#define DISPDEF 0
#define DISPWEBIF 4

int print_status(const char *device_uri, int display_style)
{
	struct device *device;
	struct status_block * status_block;
	int stat;
	const char *status_printer_strings[] = { STATUS_PRINTER_STRINGS };
	const char *status_str;

	if (device_uri == NULL && (display_style == DISPWEBIF)) {
		return 1;
	}

	/* get device */
	device = device_get_device(device_uri);
	if (device == NULL) {
		if (device_uri == NULL) {
			log_error("Can't find any supported HP devices.");
			return 2;
		} else {
			log_error("Can't find device `%s'", device_uri);
			return 3;
		}
	}
	log_info("Using device `%s'", device->uri);

	if (device_query(device) != 0) {
		log_error("Can't query device");
		return -1;
	}
	log_debug("device id: %s", device->id);

	if (device->md->status_type == HPMUD_STATUSTYPE_NA) {
		log_error("Status unsupported on device `%s'", device->uri);
		return 4;
	}
	/* TODO add support for other status reporting methods */
	if (device->md->status_type != HPMUD_STATUSTYPE_SFIELD
			&& device->md->status_type != HPMUD_STATUSTYPE_VSTATUS) {
		log_error("Status not yet implemented for device `%s'. Bummer :/",
				device->model);
		return 5;
	}

	if (device_uri == NULL && display_style != DISPWEBIF) {
		printf("%s (%s)\n", device->model, device->uri);
	}

	status_block = status_parseStatus(device->id);
	stat = status_block->status_code;
	log_debug("status_code: %u", stat);

	if (stat >= STATUS_PRINTER_IDLE && stat <= STATUS_PRINTER_PEN_CHECKING) {
		status_str = status_printer_strings[stat - STATUS_PRINTER_BASE];
	} else {
		status_str = STATUS_PRINTER_UNKNOWN;
	}
	if (display_style == DISPWEBIF) {
		printf("%d\n", stat);
	} else {
		printf("status: %s\n", status_str);
	}

	/* clean up */
	my_free(status_block);
	device_free(device);

	return 0;
}

void help(const char *pname)
{
	printf("Usage: %s [OPTION]...\n"
			"Display current status for supported HPLIP printers.\n"
			"\n"
			"  -d, --device=URI          use device with URI URI\n"
			"      --web-interface       generate output for hp-utils web interface\n"
			"  -l, --log-level=LEVEL     set log level to LEVEL\n"
#ifndef NDEBUG
			"                            {none, error, warning, debug, debug2, all}\n"
#else
			"                            {none, error, all}\n"
#endif
			"      --help                display this help and exit\n"
			"      --version             output version information and exit\n",
			pname);
}

int main(int argc, char **argv)
{
	int c;
	char *device_uri = NULL;
	int display_style = DISPDEF;

	log_program = SELF;
	log_level = LOG_LEVEL_ERROR;

	while (1) {
		static struct option long_options[] = {
			{ "device",			required_argument,	0, 'd' },
			{ "web-interface",	no_argument,		0, 130 },
			{ "log-level",  	required_argument,  0, 'l' },
            { "help",       	no_argument,        0, 250 },
            { "version",    	no_argument,        0, 251 },
			{ 0, 0, 0, 0 }
		};
		int option_index = 0;

		c = getopt_long(argc, argv, "d:l:", long_options, &option_index);

		if (c == -1) {
			break;
		}

		switch (c) {
			case 'd':
				device_uri = optarg;
				break;
			case 130:
				display_style = DISPWEBIF;
				break;
			case 'l':
				if (log_parse(optarg, &log_level) != 0) {
					helpinfo(argv[0], "illegal clean level `%s'", optarg);
					return 1;
				}
				break;
			case 250:
				help(argv[0]);
				return 0;
			case 251:
				print_version();
				return 0;
			case '?':
				helpinfo(argv[0], NULL);
				return 1;
		}
	}

	if (optind < argc) {
		helpinfo(argv[0], "extra operand `%s'", argv[optind]);
		return 1;
	}

	if (display_style != DISPWEBIF) {
		print_copy_msg();
	}

	return print_status(device_uri, display_style);
}
