/*
 * Logging facility
 * Copyright (c) 2008 Michael Denk <code@michaeldenk.de>
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdarg.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "logger.h"

const char *log_program = NULL;
int log_level = LOG_LEVEL_ERROR;

int log_parse(const char *s, int *lvl)
{
	if (strcmp(s, LOG_LEVEL_NONE_STR) == 0) {
		*lvl = LOG_LEVEL_NONE;
	} else if (strcmp(s, LOG_LEVEL_ERROR_STR) == 0) {
		*lvl = LOG_LEVEL_ERROR;
#ifndef NDEBUG
	} else if (strcmp(s, LOG_LEVEL_WARNING_STR) == 0) {
		*lvl = LOG_LEVEL_WARNING;
	} else if (strcmp(s, LOG_LEVEL_INFO_STR) == 0) {
		*lvl = LOG_LEVEL_INFO;
	} else if (strcmp(s, LOG_LEVEL_DEBUG_STR) == 0) {
		*lvl = LOG_LEVEL_DEBUG;
	} else if (strcmp(s, LOG_LEVEL_DEBUG2_STR) == 0) {
		*lvl = LOG_LEVEL_DEBUG2;
#endif
	} else if (strcmp(s, LOG_LEVEL_ALL_STR) == 0) {
		*lvl = LOG_LEVEL_ALL;
	} else {
		return 1;
	}
	return 0;
}

#ifndef NDEBUG
void _log(int level, const char *file, int line, const char *fmt, ...)
#else
void _log(int level, const char *fmt, ...)
#endif
{
	FILE *stream;
	va_list args;
	const char *level_str = NULL;

	if (level > log_level) {
		return;
	}

	if (log_level < LOG_LEVEL_DEBUG) {
		stream = stderr;
	} else {
		stream = stdout;
		fprintf(stream, "%s[%d]: ", log_program, getpid());
	}

	switch (level) {
		case LOG_LEVEL_ERROR:
			level_str = LOG_LEVEL_ERROR_STR;
			break;
		case LOG_LEVEL_WARNING:
			level_str = LOG_LEVEL_WARNING_STR;
			break;
		case LOG_LEVEL_INFO:
			level_str = LOG_LEVEL_INFO_STR;
			break;
		case LOG_LEVEL_DEBUG:
			level_str = LOG_LEVEL_DEBUG_STR;
			break;
		case LOG_LEVEL_DEBUG2:
			level_str = LOG_LEVEL_DEBUG2_STR;
			break;
	}
	fprintf(stream, "%s: ", level_str);

	va_start(args, fmt);
	vfprintf(stream, fmt, args);
	va_end(args);

#ifdef NDEBUG
	fprintf(stream, "\n");
#else
	if (log_level < LOG_LEVEL_DEBUG) {
		fprintf(stream, "\n");
	} else {
		fprintf(stream, ": %s,%d\n", file, line);
	}
#endif
}

#ifndef NDEBUG
void _log_data(const char *_buf, size_t size)
{
	const unsigned char *buf = (const unsigned char *) _buf;
	size_t i;

	if (log_level < LOG_LEVEL_DEBUG) {
		return;
	}

	for (i = 0; i < size; i++) {
		if (i % 16 == 0) {
			printf("debug: %08x  ", i);
		}
		printf("%02x ", buf[i]);
		if (i % 16 == 7 || i % 16 == 15) {
			printf(" ");
		}
#if 0
		int j;
		if (i+1 >= size || i % 16 == 15) {
			for (j = i-15; j <= i; j++) {
				if (isprint(buf[j])) {
					printf("%c", buf[j]);
				} else {
					printf(".");
				}
			}
		}
#endif
		if (i % 16 == 15) {
			printf("\n");
		}
	}
	if ((i-1) % 16 != 15) {
		printf("\n");
	}
}
#endif
