/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <netinet/tcp.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>

#include "maint.h"
#include "logger.h"
#include "pml.h"
#include "device.h"

static int socket_connect(const char *host, int port)
{
	struct hostent *hp;
	struct sockaddr_in addr;
	int on = 1, sock;

	if ((hp = gethostbyname(host)) == NULL) {
		log_error("gethostbyname: %s", hstrerror(h_errno));
		return -1;
	}

	bcopy(hp->h_addr, &addr.sin_addr, hp->h_length);
	addr.sin_port = htons(port);
	addr.sin_family = AF_INET;
	sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (const char *) &on,
			sizeof(int));
	if (sock == -1) {
		log_error("setsockopt: %s", strerror(errno));
		return -1;
	}
	if (connect(sock, (struct sockaddr *) &addr,
				sizeof(struct sockaddr_in)) == -1) {
		log_error("connect: %s", strerror(errno));
		return -1;
	}
	return sock;
}

static int write_printserv(const char *host, int port, const char *data,
		size_t len)
{
	int fd;
	int ret_code = 0;

	log_data(data, len);

	fd = socket_connect(host, port);
	if (fd < 0) {
		return 1;
	}

	if (write(fd, data, len) < 0) {
		ret_code = 1;
	}

	shutdown(fd, SHUT_RDWR);
	close(fd);

	return ret_code;
}

static int write_data(int mode, struct device *d, const char *host, int port,
		const char *data, size_t len)
{
	switch (mode) {
		case MODE_HPMUD: return device_printData(d, data, len);
		case MODE_PRINTSERV: return write_printserv(host, port, data, len);
		default: log_debug("Illegal mode: %d", mode); return 1;
	}
}

int maint_clean_pcl(int lvl, int mode, struct device *d, const char *host,
		int port)
{
	char buf[256];
	size_t len;
	struct pml pml = { PML_CLEAN_OID, PML_CLEAN_TYPE };
	int style = 1;
	int value;

	switch (lvl) {
		case 1: value = PML_CLEAN_CLEAN; break;
		case 2: value = PML_CLEAN_PRIME; break;
		case 3: value = PML_CLEAN_WIPE_AND_SPIT; break;
		default: log_debug("Illegal clean level: %d", lvl); return 1;
	}

	len = pml_buildEmbeddedPML(buf, sizeof(buf), &pml, value, style);

	return write_data(mode, d, host, port, buf, len);
}

int maint_clean_lidil(int lvl, int mode, struct device *d, const char *host,
		int port)
{
	char reset[] = { 0x24, 0x00, 0x10, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x02, 0xff, 0xff, 0xff, 0xff, 0x24 };
	char clean1[] = { 0x24, 0x00, 0x10, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		0x00, 0x02, 0xff, 0xff, 0xff, 0xff, 0x24 };
	char clean2[] = { 0x24, 0x00, 0x10, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		0x00, 0x03, 0xff, 0xff, 0xff, 0xff, 0x24 };
	char clean3[] = { 0x24, 0x00, 0x10, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		0x00, 0x04, 0xff, 0xff, 0xff, 0xff, 0x24 };
	char buf[256];

	memcpy(buf, reset, 16);
	switch (lvl) {
		case 1: memcpy(buf + 16, clean1, 16); break;
		case 2: memcpy(buf + 16, clean2, 16); break;
		case 3: memcpy(buf + 16, clean3, 16); break;
		default: log_debug("Illegal clean level: %d", lvl); return 1;
	}

	return write_data(mode, d, host, port, buf, 32);
}
