/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "pml.h"
#include "pcl.h"
#include "util.h"
#include "logger.h"

static void pack_int(char *out, int val)
{
	*out++ = (val >> 24) & 0xff;
	*out++ = (val >> 16) & 0xff;
	*out++ = (val >> 8) & 0xff;
	*out++ = (val >> 0) & 0xff;
}

static int unpack_int_be(const char *data)
{
	int i;

	i = (data[0] & 0xff) >> 24;
	i |= (data[1] & 0xff) >> 16;
	i |= (data[2] & 0xff) >> 8;
	i |= (data[3] & 0xff) >> 0;

	return i;
}

size_t pml_buildEmbeddedPMLSetPacket(char *buf, size_t size, struct pml *pml, int value)
{
	if (size < 4) {
		return 0;
	}

	strcpy(buf, "PML\x20");
	return 4 + pml_buildPMLSetPacket(buf + 4, size - 4, pml, value);
}

size_t pml_buildPMLSetPacket(char *out, size_t size, struct pml *pml, int value)
{
	char buf[64];
	char *pbuf = buf;
	char data[64];
	char *b;
	int len;
	int i;
	int data_len;
	int data_size = 0;
	char *pout = out;

	log_debug2("oid=%s, type=%d, value=%d", pml->oid, pml->type, value);

	*pout++ = SET_REQUEST;
	*pout++ = TYPE_OBJECT_IDENTIFIER;

	len = (strlen(pml->oid)+1)/2;
	*pout++ = len;

	b = strtok(pml->oid, ".");
	while (b != NULL) {
		b = strip(b);
		*pout++ = atoi(b);
		b = strtok(NULL, ".");
	}

	switch (pml->type) {
		case TYPE_ENUMERATION:
		case TYPE_SIGNED_INTEGER:
		case TYPE_COLLECTION:
			data_len = 4;
			pack_int(pbuf, value);
			if (value > 0) {
				for ( ; *pbuf == 0x00; data_len--, pbuf++);
			} else {
				unsigned char *foo = (unsigned char *) pbuf;
				for (i = 0; i < 3; i++) {
					if (*foo == 0xff && *(foo+1) == 0xff) {
						data_len--;
						pbuf++;
					}
				}
			}
			data[0] = pml->type;
			data[1] = data_len;
			strncpy(data+2, pbuf, data_len);
			data_size = 2+data_len;
			break;
		case TYPE_REAL:
			break;
		case TYPE_STRING:
			break;
		case TYPE_BINARY:
			break;
	}

	strncpy(pout, data, data_size);
	return 3+len+data_size;
}

size_t pml_buildEmbeddedPML(char *buf, size_t size, struct pml *pml, int value, int style)
{
	char buf2[256];
	size_t len;
	size_t (*func)(char *, size_t, const char *, size_t);

	log_debug("pml_buildEmbeddedPMLSetPacket");
	len = pml_buildEmbeddedPMLSetPacket(buf, size, pml, value);
	log_debug2("bytes=%u", len);
	log_data(buf, len);

	log_debug("pcl_buildPCLCmd");
	len = pcl_buildPCLCmd(buf2, sizeof buf2, '&', 'b', 'W', buf, len, 0);
	log_data(buf2, len);

	if (style == 1) {
		log_debug("pcl_buildEmbeddedPML2");
		func = pcl_buildEmbeddedPML2;
	} else {
		log_debug("pcl_buildEmbeddedPML");
		func = pcl_buildEmbeddedPML;
	}

	len = func(buf, size, buf2, len);
	log_data(buf, len);

	return len;
}

size_t pml_ConvertToPMLDataFormat(const char *value, size_t len, int type, char *out, size_t size)
{
	int int_val;
	char data[256];
	size_t len2 = 0;
	char *p = data;

	switch (type) {
		case TYPE_ENUMERATION:
		case TYPE_SIGNED_INTEGER:
		case TYPE_COLLECTION:
			int_val = atoi(value);
			pack_int(p, int_val); len2 = 4;
			if (int_val > 0) {
				for (; len2 > 0 && *p == 0x00; len2--, p++);
			} else {
				for (; len2 > 1 && *p == (char) 0xff && *(p+1) == (char) 0xff; len2--, p++);
			}

			break;
		case TYPE_REAL:
			break;
		case TYPE_STRING:
			len2 = sprintf(p, "%c%c%s", 0x00, 0x0e, value);
			break;
		case TYPE_BINARY:
			memcpy(p, value, len); len2 = len;
			break;
	}

	len2 = MIN(len2, size);
	memcpy(out, p, len2);
	return len2;
}

size_t pml_ConvertFromPMLDataFormat(const char *data, size_t len, int type, char *out, size_t size)
{
	size_t len2 = 0;
	int i;

	switch (type) {
		case TYPE_ENUMERATION:
		case TYPE_SIGNED_INTEGER:
		case TYPE_COLLECTION:
			if (len > 0) {
				char buf[4];
				const char *s = data;

				if (len < 4) {
					char *p = buf;
					s = buf;
					memcpy(p + 4 - len + 1, data, len);
					if (data[0] == (char) 0xff) {
						while (len < 4) {
							*p++ = 0xff;
							len++;
						}
					} else {
						while (len < 4) {
							*p++ = 0x00;
							len++;
						}
					}
				}

				i = unpack_int_be(s);
			} else {
				i = 0;
			}
			len2 = snprintf(out, size, "%d", i);
			break;
		case TYPE_REAL:
			if (len == 4) {
				/* TODO add me */
			} else {
				len2 = snprintf(out, size, "0.0");
			}
			break;
		case TYPE_STRING:
			for (i = 0; i < len; i++) {
				if (isprint(data[i])) {
					*out++ = data[i];
					len2++;
				}
			}
			break;
		case TYPE_BINARY:
			len2 = MIN(len, size);
			memcpy(out, data, len2);
			break;
		default:
			log_debug("illegal type: %d", type);
	}

	return len2;
}

void pml_PMLToSNMP(const char *oid, char *out, size_t size)
{
	if (strlen(oid) < 2) {
		log_debug("invalid oid (too short): %s", oid);
		*out = '\0';
		return;
	}

	switch (*oid) {
		case '0':
			strcpy(out, oid + 2);
			break;
		case '1':
			sprintf(out, "%s.%s.0", "1.3.6.1.4.1.11.2.3.9.4.2", oid);
			break;
		case '2':
			sprintf(out, "%s.%s", "1.3.6.1.2.1.43", oid + 2);
			break;
		case '3':
			sprintf(out, "%s.%s", "1.3.6.1.2.1.25", oid + 2);
			break;
		default:
			log_debug("invalid oid: %s", oid);
			*out = '\0';
	}
}
